const fs = require('fs-extra');
const R = require('ramda');
const yaml = require('js-yaml');
const mu = require('./misc-utils.js');

const toHours = duration => {
  const last = duration.length - 1;
  const unit = duration.charAt(last);
  return duration.substr(0, last) * { 'h': 1, 'd': 8}[unit];
}

// Log is an ordered Map, where the keys are Date objects
class Log {
  constructor(data) {
    const logMap = this.logMap = new Map();
    const logArray = this.logArray = [];

    Object.keys(data).forEach(dateStr => {
      // A log entry is an object that has a `date` property, as well as all
      // the other properties specified in the YAML file
      const d = new Date(dateStr);
      const logEntry = R.merge(
        {date: d},
        R.evolve({
            worked: toHours,
            estimate: toHours,
          },
          data[dateStr]
        )
      );
      logMap.set(d, logEntry);
      logArray.push(logEntry);
    });
  }

  first() {
    return this.logArray[0];
  }
  length() {
    return this.logArray.length;
  }
  last() {
    return this.logArray[this.length() - 1];
  }
}

class Issue {
  constructor(data, key) {
    this.id = key;
    const issueData = R.evolve({
      log: logData => new Log(logData),
    }, data)
    Object.assign(this, issueData);
  }

  // original estimate, in hours
  origEstimate() {
    return this.log.first().estimate;
  }

  // current estimate, in hours
  currEstimate() {
    return this.log.last().estimate;
  }
}
const newIssue = (data, key) => new Issue(data, key);

class Group {
  constructor(name, issueData) {
    this.name = name;
    this.issues = R.mapObjIndexed(newIssue, issueData);
    this.issueArray = R.values(this.issues);
  }

  // FIXME: eliminate redundancy here
  totalOrigEst() {
    return mu.sum(this.issueArray.map(issue => issue.origEstimate()));
  }
  totalCurrEst() {
    return mu.sum(this.issueArray.map(issue => issue.currEstimate()));
  }
}
const newGroup = (issueData, name) => new Group(name, issueData);


// FIXME: turn this into a subclass of an ordered mapping, which should
// be array-like, and let you access the members with by integer index or
// by name.
class IssueDb {
  constructor(issuesFile) {
    const data = yaml.safeLoad(fs.readFileSync(issuesFile, 'utf8'));

    this.title = R.keys(data)[0];
    const dbData = R.values(data)[0];
    this.groupsByName = R.mapObjIndexed(newGroup, dbData);
    this.groups = R.values(this.groupsByName);
  }
}

module.exports = IssueDb;
