const R = require('ramda');

const sum = R.reduce(R.add, 0);

// Given an object of objects, returns an array of objects; the new objects
// have the `name` member set with their keys from the container object.
const arrayize = obj => {
  const keys = Object.keys(obj);
  return keys.map(k => R.assoc('name', k, obj[k]));
};

// This function takes an object and a key, and returns a new object with
// the `name` property set.
const includeKeyAsName = (obj, key) => {
  return R.assoc('name', key, obj);
}

// A common pattern is to take a dictionary of objects, and include the
// keys for each object into the object itself, as the `name` property.
// For example, convert
//   { fleegle: { instrument: 'banjo' },
//     bingo:   { instrument: 'guitar' } }
// to
//   { fleegle: { name: 'fleegle', instrument: 'banjo' },
//     bingo:   { name: 'bingo',   instrument: 'guitar' } }
const setWithNames = R.mapObjIndexed(includeKeyAsName);

module.exports = {
  sum: sum,
};
